@@ -50,6 +50,7 @@
 
 #define ADD_NONCE "&nonce="
 #define ADD_OTP "&otp="
+#define ADD_HASH "&h="
 #define ADD_ID "?id="
 
 #define TEMPLATE_FORMAT_OLD 1
@@ -70,7 +71,6 @@ struct ykclient_st
   char *nonce;
   char nonce_supplied;
   int verify_signature;
-  const char *user_agent;
 };
 
 struct curl_data
@@ -88,14 +88,16 @@ struct ykclient_handle_st
   char **url_exp;
 };
 
-const char *default_url_templates[] = {
-  "http://api.yubico.com/wsapi/2.0/verify",
-  "http://api2.yubico.com/wsapi/2.0/verify",
-  "http://api3.yubico.com/wsapi/2.0/verify",
-  "http://api4.yubico.com/wsapi/2.0/verify",
-  "http://api5.yubico.com/wsapi/2.0/verify",
+static const char *default_url_templates[] = {
+  "https://api.yubico.com/wsapi/2.0/verify",
+  "https://api2.yubico.com/wsapi/2.0/verify",
+  "https://api3.yubico.com/wsapi/2.0/verify",
+  "https://api4.yubico.com/wsapi/2.0/verify",
+  "https://api5.yubico.com/wsapi/2.0/verify",
 };
 
+static const char *user_agent = PACKAGE "/" PACKAGE_VERSION;
+
 /** Initialise the global context for the library
  *
  * This function is not thread safe.  It must be invoked successfully
@@ -166,14 +168,6 @@ ykclient_init (ykclient_t ** ykc)
    */
   p->verify_signature = 0;
 
-  *ykc = p;
-
-  /*
-   * Set the User-Agent string that will be used by the CURL
-   * handles.
-   */
-  p->user_agent = PACKAGE "/" PACKAGE_VERSION;
-
   /*
    * Set the default URLs (these can be overridden later)
    */
@@ -181,6 +175,8 @@ ykclient_init (ykclient_t ** ykc)
 			  sizeof (default_url_templates) /
 			  sizeof (char *), default_url_templates);
 
+  *ykc = p;
+
   return YKCLIENT_OK;
 }
 
@@ -317,17 +313,17 @@ ykclient_handle_init (ykclient_t * ykc, ykclient_handle_t ** ykh)
       if (ykc->ca_path)
 	{
 	  curl_easy_setopt (easy, CURLOPT_CAPATH, ykc->ca_path);
-        }
+	}
 
       if (ykc->ca_info)
-        {
-          curl_easy_setopt (easy, CURLOPT_CAINFO, ykc->ca_info);
+	{
+	  curl_easy_setopt (easy, CURLOPT_CAINFO, ykc->ca_info);
 	}
 
       curl_easy_setopt (easy, CURLOPT_WRITEDATA, (void *) data);
       curl_easy_setopt (easy, CURLOPT_PRIVATE, (void *) data);
       curl_easy_setopt (easy, CURLOPT_WRITEFUNCTION, curl_callback);
-      curl_easy_setopt (easy, CURLOPT_USERAGENT, ykc->user_agent);
+      curl_easy_setopt (easy, CURLOPT_USERAGENT, user_agent);
 
       curl_multi_add_handle (p->multi, easy);
       p->easy[p->num_easy] = easy;
@@ -595,8 +591,9 @@ ykclient_rc
 ykclient_set_url_templates (ykclient_t * ykc, size_t num_templates,
 			    const char **url_templates)
 {
-  ykclient_rc ret = ykclient_set_url_bases(ykc, num_templates, url_templates);
-  if(ret == YKCLIENT_OK)
+  ykclient_rc ret =
+    ykclient_set_url_bases (ykc, num_templates, url_templates);
+  if (ret == YKCLIENT_OK)
     {
       ykc->template_format = TEMPLATE_FORMAT_OLD;
     }
@@ -605,7 +602,7 @@ ykclient_set_url_templates (ykclient_t * ykc, size_t num_templates,
 
 ykclient_rc
 ykclient_set_url_bases (ykclient_t * ykc, size_t num_templates,
-			    const char **url_templates)
+			const char **url_templates)
 {
   size_t i;
   if (num_templates > MAX_TEMPLATES)
@@ -834,40 +831,44 @@ ykclient_generate_nonce (ykclient_t * ykc, char **nonce)
   return YKCLIENT_OK;
 }
 
-static ykclient_rc ykclient_expand_new_url(const char *template,
-    const char *encoded_otp, const char *nonce, int client_id,
-    char **url_exp)
+static ykclient_rc
+ykclient_expand_new_url (const char *template,
+			 const char *encoded_otp, const char *nonce,
+			 int client_id, char **url_exp)
 {
-  size_t len = strlen(template) + strlen(encoded_otp) + strlen(ADD_OTP) + strlen(ADD_ID) + 1;
-  len += snprintf(NULL, 0, "%d", client_id);
+  size_t len =
+    strlen (template) + strlen (encoded_otp) + strlen (ADD_OTP) +
+    strlen (ADD_ID) + 1;
+  len += snprintf (NULL, 0, "%d", client_id);
 
-  if(nonce)
+  if (nonce)
     {
-      len += strlen(nonce) + strlen(ADD_NONCE);
+      len += strlen (nonce) + strlen (ADD_NONCE);
     }
 
-  *url_exp = malloc(len);
-  if(!*url_exp)
+  *url_exp = malloc (len);
+  if (!*url_exp)
     {
       return YKCLIENT_OUT_OF_MEMORY;
     }
 
-  if(nonce)
+  if (nonce)
     {
-      snprintf(*url_exp, len, "%s" ADD_ID "%d" ADD_NONCE "%s" ADD_OTP "%s",
-	  template, client_id, nonce, encoded_otp);
+      snprintf (*url_exp, len, "%s" ADD_ID "%d" ADD_NONCE "%s" ADD_OTP "%s",
+		template, client_id, nonce, encoded_otp);
     }
   else
     {
-      snprintf(*url_exp, len, "%s" ADD_ID "%d" ADD_OTP "%s", template,
-	  client_id, encoded_otp);
+      snprintf (*url_exp, len, "%s" ADD_ID "%d" ADD_OTP "%s", template,
+		client_id, encoded_otp);
     }
   return YKCLIENT_OK;
 }
 
-static ykclient_rc ykclient_expand_old_url(const char *template,
-    const char *encoded_otp, const char *nonce, int client_id,
-    char **url_exp)
+static ykclient_rc
+ykclient_expand_old_url (const char *template,
+			 const char *encoded_otp, const char *nonce,
+			 int client_id, char **url_exp)
 {
   {
     size_t len;
@@ -877,14 +878,13 @@ static ykclient_rc ykclient_expand_old_url(const char *template,
     *url_exp = malloc (len);
     if (!*url_exp)
       {
-        return YKCLIENT_OUT_OF_MEMORY;
+	return YKCLIENT_OUT_OF_MEMORY;
       }
 
-    wrote = snprintf (*url_exp, len, template,
-        client_id, encoded_otp);
+    wrote = snprintf (*url_exp, len, template, client_id, encoded_otp);
     if (wrote < 0 || (size_t) wrote > len)
       {
-        return YKCLIENT_FORMAT_ERROR;
+	return YKCLIENT_FORMAT_ERROR;
       }
   }
 
@@ -895,14 +895,12 @@ static ykclient_rc ykclient_expand_old_url(const char *template,
       size_t len;
       ssize_t wrote;
 
-      len =
-        strlen (*url_exp) + strlen (ADD_NONCE) + strlen (nonce) +
-        1;
+      len = strlen (*url_exp) + strlen (ADD_NONCE) + strlen (nonce) + 1;
       nonce_url = malloc (len + 4);	/* avoid valgrind complaint */
       if (!nonce_url)
-        {
-          return YKCLIENT_OUT_OF_MEMORY;
-        }
+	{
+	  return YKCLIENT_OUT_OF_MEMORY;
+	}
 
       /* Find the &otp= in url and insert ?nonce= before otp. Must get
        *  sorted headers since we calculate HMAC on the result.
@@ -915,26 +913,26 @@ static ykclient_rc ykclient_expand_old_url(const char *template,
        */
       otp_offset = strstr (*url_exp, ADD_OTP);
       if (otp_offset == NULL)
-        {
-          /* point at \0 at end of url in case there is no otp */
-          otp_offset = *url_exp + len;
-        }
+	{
+	  /* point at \0 at end of url in case there is no otp */
+	  otp_offset = *url_exp + len;
+	}
 
       /* break up ykc->url where we want to insert nonce */
       *otp_offset = 0;
 
       wrote = snprintf (nonce_url, len, "%s" ADD_NONCE "%s&%s", *url_exp,
-          nonce, otp_offset + 1);
+			nonce, otp_offset + 1);
       if (wrote < 0 || (size_t) wrote + 1 != len)
-        {
-          free (nonce_url);
-          return YKCLIENT_FORMAT_ERROR;
-        }
+	{
+	  free (nonce_url);
+	  return YKCLIENT_FORMAT_ERROR;
+	}
 
       free (*url_exp);
       *url_exp = nonce_url;
     }
-    return YKCLIENT_OK;
+  return YKCLIENT_OK;
 }
 
 /** Expand URL templates specified with set_url_templates
@@ -969,27 +967,33 @@ ykclient_expand_urls (ykclient_t * ykc, ykclient_handle_t * ykh,
       return YKCLIENT_HANDLE_NOT_REINIT;
     }
 
-  /* URL-encode the OTP */
-  encoded_otp = curl_easy_escape (ykh->multi, yubikey, 0);
-
   for (i = 0; i < ykc->num_templates; i++)
     {
       ykclient_rc ret;
-      if(ykc->template_format == TEMPLATE_FORMAT_OLD)
-        {
-	  ret = ykclient_expand_old_url(ykc->url_templates[i],
-	      encoded_otp, nonce, ykc->client_id, &ykh->url_exp[i]);
-        }
+
+      if (!encoded_otp)
+	{
+	  /* URL-encode the OTP */
+	  encoded_otp = curl_easy_escape (ykh->easy[i], yubikey, 0);
+	}
+
+      if (ykc->template_format == TEMPLATE_FORMAT_OLD)
+	{
+	  ret = ykclient_expand_old_url (ykc->url_templates[i],
+					 encoded_otp, nonce, ykc->client_id,
+					 &ykh->url_exp[i]);
+	}
       else
 	{
-	  ret = ykclient_expand_new_url(ykc->url_templates[i],
-	      encoded_otp, nonce, ykc->client_id, &ykh->url_exp[i]);
+	  ret = ykclient_expand_new_url (ykc->url_templates[i],
+					 encoded_otp, nonce, ykc->client_id,
+					 &ykh->url_exp[i]);
+	}
+      if (ret != YKCLIENT_OK)
+	{
+	  out = ret;
+	  goto finish;
 	}
-      if(ret != YKCLIENT_OK)
-        {
-          out = ret;
-          goto finish;
-        }
       if (ykc->key && ykc->keylen)
 	{
 	  if (!signature)
@@ -1027,7 +1031,7 @@ ykclient_expand_urls (ykclient_t * ykc, ykclient_handle_t * ykh,
 	      res2 = base64_encode_blockend (&b64dig[res], &b64);
 	      b64dig[res + res2 - 1] = '\0';
 
-	      signature = curl_easy_escape (ykh->multi, b64dig, 0);
+	      signature = curl_easy_escape (ykh->easy[i], b64dig, 0);
 	    }
 
 	  /* Create new URL with signature ( h= ) appended to it . */
@@ -1036,7 +1040,6 @@ ykclient_expand_urls (ykclient_t * ykc, ykclient_handle_t * ykh,
 	    size_t len;
 	    ssize_t wrote;
 
-#define ADD_HASH "&h="
 	    len =
 	      strlen (ykh->url_exp[i]) + strlen (ADD_HASH) +
 	      strlen (signature) + 1;
@@ -1244,6 +1247,13 @@ ykclient_request_send (ykclient_t * ykc, ykclient_handle_t * ykh,
 	      continue;
 	    }
 
+	  /* if we get anything other than CURLE_OK we throw away this result */
+	  if (msg->data.result != CURLE_OK)
+	    {
+	      out = YKCLIENT_CURL_PERFORM_ERROR;
+	      continue;
+	    }
+
 	  curl_easy = msg->easy_handle;
 
 	  curl_easy_getinfo (curl_easy, CURLINFO_PRIVATE, (char **) &data);
@@ -1460,7 +1470,7 @@ ykclient_verify_otp_v2 (ykclient_t * ykc_in,
 
   if (urlcount != 0 && *urls != 0)
     {
-      if(strstr(urls[0], ADD_OTP "%s"))
+      if (strstr (urls[0], ADD_OTP "%s"))
 	{
 	  ykclient_set_url_templates (ykc, urlcount, urls);
 	}
